<?php
/**
 * @name  Model_egidescan
 * @description Classe héritant de "Model_esia" gèrent tout les accès à la base de donnée du plugin Server
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 08/11/2012
 */
include_once SNMP_CLASS;
include_once DEVICE_CLASS;
include_once MVCMODEL;
include_once 'libs/egide_utils.libs.php';

class Model_egidescan extends Model_esia
{
	const NEW_TASK='new';
	const PENDING_TASK='pending';
	const PROCESSING_TASK='processing';
	const TERMINATED_TASK='terminated';
	const ERROR_TASK='error';

	const PLUGIN_CRON_START = 'CRON_EGIDE_SCAN_START';
//  const BOX_FILTER_REGEX = '20.{6}86';            // Pour  Unity x86
    const BOX_FILTER_REGEX = '.*';                  // Toutes les gesa
	//const BOX_FILTER_REGEX = '20.{6}[19]+0';        //10 ou 90, pour box Egide


	// vulscan_json
	public static function getScoreHtmlColor($score_value)
	{
		$colors = array("#00c400","#00e020","#00f000","#d1ff00","#ffe000","#ffcc00","#ffbc10","#ff9c20","#ff8000","#ff0000");
		$score_value = floor($score_value); //On arrondi en dessous
		return $colors[$score_value];
	}

	public static function existIp($dbconnection, $ip=NULL)
	{
		$sql = "SELECT COUNT(*) as detect_ip_adress
				                        FROM esia_device WHERE device_ip_address = :ip;";

		$params = array(
			':ip' => $ip
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$access = $query->fetch(PDO::FETCH_OBJ);
		$detect_ip_adress = $access->detect_ip_adress;
		if(empty($detect_ip_adress))
		{
			$detect_ip_adress = 'NONE';
		}

		return $detect_ip_adress;
	}

	public static function acquitVulnerabilitiesFromServices($dbconnection,$user_obj,$cve_list=NULL,$include_cve=TRUE)
	{
		//Préparation et validation du parameter
		$in_request = generateInParams('WHERE',$cve_list,'detect_eg_service_id',$include_cve);
		$params = $in_request['param'];
		$where = $in_request['sql'];

		$params[':user']= $user_obj->getName();

		$sql = "
				UPDATE egide_vulnerability_detect
				SET detect_state = 'ACQUIT',detect_username = :user,detect_date_update_state = now()
				$where";

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$params);
	}

	public static function acquitVulnerabilitiesFromNodes($dbconnection,$user_obj,$node_list,$cve_list=NULL,$include_cve=TRUE)
	{
		//Préparation et validation du parameter
		$params[':user']= $user_obj->getName();

		$in_request = generateInParams('WHERE',$node_list,'eg_device_id',TRUE);
		$param = $in_request['param'];
		$sql_devices = $in_request['sql'];
		$params = array_merge($params,$param);

		$in_request = generateInParams('WHERE',$cve_list,'vul_list_id',$include_cve);
		$param = $in_request['param'];
		$sql_vul = $in_request['sql'];
		$params = array_merge($params,$param);

		$sql_vul = "
				AND detect_vul_id IN(
					SELECT vul_id FROM egide_vulnerability
					$sql_vul
				)
			";

		$sql = "
				UPDATE egide_vulnerability_detect
				SET detect_state = 'ACQUIT',detect_username = :user,detect_date_update_state = now()
				WHERE detect_eg_service_id IN(
						SELECT eg_service_id FROM egide_services
						$sql_devices $sql_vul
				);";

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$params);
	}

	public static function unacquitVulnerabilitiesDetect($dbconnection,$cve_list=NULL,$include_cve=TRUE)
	{
		//Préparation et validation du parameter
		$in_request = generateInParams('AND',$cve_list,'detect_id',$include_cve);
		$params = $in_request['param'];
		$sql_vul = $in_request['sql'];

		$sql = "
				UPDATE egide_vulnerability_detect
				SET detect_state = 'ACTIVE',detect_username = NULL,detect_date_update_state = NULL
				WHERE detect_eg_service_id IN(
						SELECT eg_service_id FROM egide_services
						WHERE detect_state = 'ACQUIT' $sql_vul
				);";

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$params);
	}

// 	public static function getDeviceEsia($dbconnection, $id=NULL)
// 	{
// 		$sql = "SELECT device_name as dv_name, device_type as dv_type, device_ip_address as dv_ip FROM esia_device WHERE device_id=?";

// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query,array($id));
// 		$arr = array();
// 		while($row=$query->fetch(PDO::FETCH_OBJ))
// 		{
// 			$arr=array('dv_name'=>$row->dv_name,
// 				'dv_type'=>$row->dv_type,
// 				'dv_ip'=>$row->dv_ip
// 			);
// 		}
// 		return $arr;
// 	}

	public static function getEgideBoxList($dbconnection,$userid)
	{
		$sql = "SELECT device_id,device_name FROM esia_device";
		$sql.= " WHERE device_id IN(SELECT gesa_device_id FROM gesa_info WHERE gesa_sn::varchar ~ '".Model_egidescan::BOX_FILTER_REGEX."')";
		$sql.= " AND device_id IN ( SELECT * FROM getUserAccessNode(:userid)) ORDER BY device_name ASC";
		$query = $dbconnection->prepare($sql);
		$arr = array(':userid'=>$userid);
		$query = Model_esia::executeStatement($query,$arr);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$retRow = array();
			$retRow['id'] = $row['device_id'];
			$retRow['name'] = $row['device_name'];
			$ret[] = $retRow;
		}
		return $ret;
	}

	//child peut être une box ou un groupe de noeud
	private static function generateTreeViewBranch($group_list,$parent_id,$child)
	{
		if( empty($parent_id))
		{
			return $child;
		}

		//Generate new treeview
		$parent = $group_list[$parent_id];
		$parent['type'] = 'group';
		$parent['childs'][$child['id']] = $child;

		if( empty($parent['parent']) )
		{
			unset($parent['parent']);
			return $parent;
		}

		$parent_id = $parent['parent'];
		unset($parent['parent']);
		return self::generateTreeViewBranch($group_list,$parent_id,$parent);
	}

	private static function addBranchToTreeview($treeview,$branch)
	{
		if(empty($treeview))
		{
			$treeview = array();
		}
		if( empty($treeview[$branch['id']]))
		{
			$treeview[$branch['id']] = $branch;
		}
		else
		{
			foreach($branch['childs'] as $child)
			{
				$treeview[$branch['id']]['childs'] = self::addBranchToTreeview($treeview[$branch['id']]['childs'],$child);
			}
		}
		return $treeview;
	}

	private static function calculChild(&$treeview)
	{
		$treeview = array_values($treeview);
		foreach($treeview as $key => $branch)
		{
			if( !empty($branch['childs']))
			{
				$treeview[$key]['child'] = count($branch['childs']);
				self::calculChild($branch['childs']);
			}
			else
			{
				$treeview[$key]['child'] = 0;
			}
		}
	}

	private static function searchBranch($treeview,$nodeid)
	{
		foreach($treeview as $id => $branch)
		{
			if( $id == $nodeid )
			{
				return $branch['childs'];
			}
			else
			{
				$found = self::searchBranch($branch['childs'],$nodeid);
				if( $found !== FALSE )
				{
					return $found;
				}
			}
		}
		return FALSE;
	}

	public static function getEgideBoxTreeView($dbconnection,$userid,$nodeid)
	{
		//Récupération de tous les groupes de noeuds
		$sql = "
					WITH accessgroup AS (
						SELECT device_group_id,esia_device_group.device_group_name,device_group_parent
						FROM esia_device_group
						LEFT JOIN esia_device_group_in ON device_group_id = group_in_device_group_id
						LEFT JOIN esia_device ON device_id = group_in_device_id
						WHERE device_id IN(SELECT gesa_device_id FROM gesa_info WHERE gesa_sn::varchar ~ '".Model_egidescan::BOX_FILTER_REGEX."')
						AND (device_id IN ( SELECT * FROM getUserAccessNode(:userid)) OR device_id IS NULL)
						GROUP BY device_group_id,device_group_parent
					)
					SELECT device_group_id,device_group_name,device_group_parent
					FROM accessgroup
					UNION
					SELECT device_group_id,device_group_name,device_group_parent
					FROM esia_device_group
					WHERE device_group_id IN (SELECT device_group_parent FROM accessgroup)
					AND device_group_id NOT IN (SELECT device_group_id  FROM accessgroup);
			";

		$statement = $dbconnection->prepare($sql);
		$response = Model_esia::executeStatement($statement,array(':userid'=>$userid));
		$group_list = array();
		while($queryRow = $response->fetch(PDO::FETCH_OBJ))
		{
			$retRow['id'] = $queryRow->device_group_id;
			$retRow['name'] = $queryRow->device_group_name;
			$retRow['parent'] = $queryRow->device_group_parent;
			$group_list[$retRow['id']] = $retRow;
		}

		//On récupére les box et leur groupe parents
		$sql = "
				SELECT device_id,device_name,group_in_device_group_id
				FROM esia_device LEFT JOIN esia_device_group_in
				ON (device_id = group_in_device_id)
				WHERE device_id IN(SELECT gesa_device_id FROM gesa_info WHERE gesa_sn::varchar ~ '".Model_egidescan::BOX_FILTER_REGEX."')
				AND device_id IN ( SELECT * FROM getUserAccessNode(:userid));
			";

		$statement = $dbconnection->prepare($sql);
		$response = Model_esia::executeStatement($statement,array(':userid'=>$userid));
		$egide_boxes = array();
		while($queryRow = $response->fetch(PDO::FETCH_ASSOC))
		{
			if( !isset($egide_boxes[ $queryRow['device_id']]))
			{
				$egideRow['id'] = $queryRow['device_id'];
				$egideRow['name'] = $queryRow['device_name'];
				$egideRow['type'] = 'box';
				$egideRow['group'] = array();
				$egide_boxes[ $queryRow['device_id']] = $egideRow;
			}
			$egide_boxes[$queryRow['device_id']]['group'][] = $queryRow['group_in_device_group_id'];
		}

		$treeview = array();
		foreach($egide_boxes as $box)
		{
			$box_group_list = $box['group'];
			unset($box['group']);
			foreach($box_group_list as $group_id)
			{
				$branch = self::generateTreeViewBranch($group_list,$group_id,$box);
				$treeview = self::addBranchToTreeview($treeview,$branch);
			}
		}

		if( $nodeid != NULL )
		{
			$treeview = self::searchBranch($treeview,$nodeid);
		}
		self::calculChild($treeview);
		return $treeview;
	}

	private function searchBox($treeview)
	{
		$box_list = array();
		foreach($treeview as $row)
		{
			if( $row['type'] =='box')
			{
				$box_list[$row['id']] = array('id'=>$row['id'],'name'=>$row['name']);
			}
			else
			{
				$box_list = array_merge($box_list,self::searchBox($row['childs']));
			}
		}
		return $box_list;
	}

	public function getEgideBoxListOfGroup($dbconnection,$userid,$nodeid)
	{
		$treeview = self::getEgideBoxTreeView($dbconnection,$userid,$nodeid);
		$ret = self::searchBox($treeview);
		return array_values($ret);
	}

	public static function getConfiguratedEgideBoxIfaces($dbconnection,$node_id)
	{
		$sql = "SELECT vul_iface_id,vul_iface_agent_id,vul_iface_local_name,vul_iface_label,vul_type_id,vul_type_name,vul_type_factor,vul_type_color";
		$sql.= " FROM vulnerability_iface INNER JOIN vulnerability_type";
		$sql.= " ON(vul_iface_type_id = vul_type_id)";
		$sql.= " WHERE vul_iface_agent_id = :node_id";

		$arr = array(':node_id'=>$node_id);
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$arr);
		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$retRow = array();
			$retRow['id'] = $row['vul_iface_id'];
			$retRow['label'] = $row['vul_iface_label'];
			$retRow['local_name'] = $row['vul_iface_local_name'];
			$retRow['type_name'] = $row['vul_type_name'];
			$retRow['type_id'] = $row['vul_type_id'];
			$retRow['type_factor'] = $row['vul_type_factor'];
			$retRow['type_color'] = $row['vul_type_color'];
			$retRow['agent_id'] = $row['vul_iface_agent_id'];
			$ret[] = $retRow;
		}
		return $ret;
	}

	public static function getEgideIfacesType($dbconnection)
	{
		$sql = 'SELECT * from vulnerability_type ORDER BY vul_type_factor DESC';
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$retRow = array();
			$retRow['id'] = $row['vul_type_id'];
			$retRow['name'] = $row['vul_type_name'];
			$retRow['factor'] = $row['vul_type_factor'];
			$retRow['color'] = $row['vul_type_color'];
			$ret[] = $retRow;
		}
		return $ret;
	}

	public static function addEgideIfaceConfiguration($dbconnection,$box_id,$local_name,$label,$type)
	{
		$sql = 'INSERT INTO vulnerability_iface (vul_iface_agent_id,vul_iface_local_name,vul_iface_label,vul_iface_type_id)';
		$sql.= ' VALUES(:agent,:local,:label,:type) RETURNING vul_iface_id';

		$query = $dbconnection->prepare($sql);
		$arr[':agent'] = $box_id;
		$arr[':local'] = $local_name;
		$arr[':label'] = $label;
		$arr[':type'] = $type;
		$query = Model_esia::executeStatement($query,$arr);
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->vul_iface_id;
	}

	public static function updateEgideIfaceConfiguration($dbconnection,$box_id,$local_name,$label,$type)
	{
		$sql = 'UPDATE vulnerability_iface SET vul_iface_label = :label,vul_iface_type_id = :type';
		$sql.= ' WHERE vul_iface_agent_id = :agent AND vul_iface_local_name = :local';

		$query = $dbconnection->prepare($sql);
		$arr[':agent'] = $box_id;
		$arr[':local'] = $local_name;
		$arr[':label'] = $label;
		$arr[':type'] = $type;
		$query = Model_esia::executeStatement($query,$arr);
	}

	public static function getIpDevice($dbconnection, $name=NULL)
	{
		$sql = "SELECT device_ip_address as dv_ip FROM esia_device WHERE device_name=?";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($name));
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr=array(
				'dv_ip'=>$row->dv_ip
			);
		}
		return $arr;
	}

	public static function notifyScanningError($dbconnection,$scan_id,$message)
	{
		$scan_info = Model_egidescan::getScanById($dbconnection,$scan_id);
		$start = $scan_info['next_exec'];
		$end = date("Y-m-d H:i:s");
		$result['message'] = $message;
		self::addScanHistory($dbconnection,$start,$end,$scan_id,$result,Model_egidescan::ERROR_TASK);
		self::updateScanningTaskStatus($dbconnection,$scan_id,Model_egidescan::ERROR_TASK);
	}

	public static function addScanHistory($dbconnection, $start,$end,$scan_id,$result,$scan_state=NULL)
	{
		$sql = "INSERT INTO public.vulnerability_scanner_history (scan_date_start, scan_date_end,scan_id,scan_result,scan_state) VALUES (:scan_start, :scan_end, :scan_id, :result,:scan_state);";
		$params[':scan_start'] = $start;
		$params[':scan_end'] = $end;
		$params[':scan_id'] = $scan_id;
		$params[':scan_state'] = $scan_state;
		$params[':result'] = json_encode($result);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
	}

	/*
	 public static function addScanHistory($dbconnection, $t0=NULL, $t1=NULL, $scan_id_device=NULL , $devices_scanned = NULL)
	 {
	 $sql = "INSERT INTO public.vulnerability_scanner_history (scan_date_start, scan_date_end,scan_id,scan_result) VALUES (:t0, :t1, :scan_id_device, :devices_scanned);";

	 $params = array(
	 ':t0' => $t0,	modifié des deux côtés :  plugin/egidescan/model_egidescan.php

	 ':t1' => $t1,
	 ':scan_id_device' => $scan_id_device,
	 ':devices_scanned' => $devices_scanned
	 );

	 $query = $dbconnection->prepare($sql);
	 $query = Model_esia::executeStatement($query, $params);
	 }
	 */
	// egide_base_de_donnees

// 	public static function addProvider($dbconnection, $prov=NULL)
// 	{
// 		$sql = "INSERT INTO public.egide_provider(provider_name) VALUES (:prov)";
// 		$params = array(
// 			':prov' => $prov
// 		);

// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query, $params);
// 	}

// 	public static function getProviderId($dbconnection, $prov=NULL)
// 	{
// 		$sql = "SELECT provider_id as nbr
// 				        FROM egide_provider
// 				        WHERE provider_name=:prov;";

// 		$params = array(
// 			':prov' => $prov
// 		);

// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query, $params);
// 		$access = $query->fetch(PDO::FETCH_OBJ);
// 		$nbr = $access->nbr;
// 		if(empty($nbr))
// 		{
// 			$nbr = 'NONE';
// 		}

// 		return $nbr;
// 	}

	public static function getVulnerability($dbconnection)
	{
		$sql = "SELECT vul_list_id as vul_id_1 FROM public.egide_vulnerability;";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$row=$query->fetchAll(PDO::FETCH_ASSOC);
		if(empty($row))
		{
			$row = 'NONE';
		}

		return $row;
	}

// 	public static function addVulnerability($dbconnection, $provId=NULL,$vul=NULL,$vulDescr=NULL, $vulUrl=NULL,$vulDatePub=NULL,$vulScore=NULL,$vulSeverity=NULL,$vulComplexity=NULL)
// 	{
// 		$sql = "INSERT INTO public.egide_vulnerability(vul_provider_id, vul_list_id, vul_description, vul_url, vul_date_publishing, vul_scoring, vul_severity, vul_attack_complexity)
//                                         VALUES (:provId, :vul, :vulDescr, :vulUrl, :vulDatePub, :vulScore, :vulSeverity, :vulComplexity);";
// 		$params = array(
// 			':provId' =>$provId,
// 			':vul' =>$vul,
// 			':vulDescr' =>$vulDescr,
// 			':vulUrl' =>$vulUrl,
// 			':vulDatePub' =>$vulDatePub,
// 			':vulScore' =>$vulScore,
// 			':vulSeverity' =>$vulSeverity,
// 			':vulComplexity' =>$vulComplexity
// 		);

// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query, $params);
// 	}

	public static function getScore($dbconnection, $cve_id=NULL)
	{
		$sql = "SELECT vul_scoring as score FROM public.egide_vulnerability WHERE vul_list_id=:cve_id;";

		$params = array(
			':cve_id' => $cve_id
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$access = $query->fetch(PDO::FETCH_OBJ);
		$score = $access->score;
		if(empty($score))
		{
			$score = 'NONE';
		}

		return $score;
	}

// 	public static function getVulHistotyId($dbconnection)
// 	{
// 		$sql = "SELECT detect_device_id as diviceid, detect_vul_id as vulid FROM public.egide_vulnerability_detect;";

// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query);
// 		$row=$query->fetchAll(PDO::FETCH_ASSOC);
// 		if(empty($row))
// 		{
// 			$row = 'NONE';
// 		}

// 		return $row;
// 	}

// 	public static function getVulId($dbconnection,$vul=NULL)
// 	{
// 		$sql = "SELECT vuld_id as vuld_id_db FROM public.egide_vulnerability WHERE vul_list_id = :vul;";
// 		$params = array(
// 			':vul' =>$vul
// 		);
// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query, $params);
// 		$row=$query->fetch(PDO::FETCH_OBJ);

// 		$vuld_id_db = $row->vuld_id_db;
// 		if(empty($vuld_id_db))
// 		{
// 			$vuld_id_db = 'NONE';
// 		}
// 		return $vuld_id_db;
// 	}

// 	public static function addVulHistory($dbconnection, $deviceId=NULL,$vulId=NULL,$dateDetect=NULL)
// 	{
// 		$sql = "INSERT INTO public.egide_vulnerability_detect(
// 	                                detect_device_id, detect_vul_id, detect_date)
// 	                                VALUES (:deviceId, :vulId, :dateDetect);";
// 		$params = array(
// 			':deviceId' => $deviceId,
// 			':vulId' => $vulId,
// 			':dateDetect' => $dateDetect
// 		);

// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query, $params);
// 	}

// 	public static function getEgide_vulnerability($dbconnection)
// 	{
// 		$sql = "SELECT vul_list_id as id1, vul_description as descrp1, vul_scoring as score1 FROM egide_vulnerability;";

// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query);
// 		$row=$query->fetchAll(PDO::FETCH_ASSOC);
// 		if(empty($row))
// 		{
// 			$row = 'NONE';
// 		}

// 		return $row;
// 	}

	public static function getDevicesOfAgent($dbconnection,$agent_id,$user_id=NULL)
	{
		$sql = "
		SELECT DISTINCT eg_service_id,eg_device_id
		FROM vulnerability_iface
		INNER JOIN egide_service_access ON (vul_iface_id = eg_access_iface_id)
		INNER JOIN egide_services ON (eg_access_service_id = eg_service_id)
		WHERE vul_iface_agent_id = :agent_id
		";

		$params[':agent_id'] = $agent_id;
		if( !empty($user_id) )
		{
			$params[':userid'] = $user_id;
			$sql.= ' AND vul_iface_agent_id IN ( SELECT * FROM getUserAccessNode(:userid))';
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$row=$query->fetchAll(PDO::FETCH_ASSOC);
		return $row;
	}

	private static function getTreeViewBox($treeview)
	{
		$box_list = array();
		foreach($treeview as $branch)
		{
			if( $branch['type'] == 'box')
			{
				$box_list[] = $branch;
			}
			if( !empty($branch['childs']) )
			{
				$box_list = array_merge($box_list,self::getTreeViewBox($branch['childs']));
			}
		}
		return $box_list;
	}

	public static function getAgentsOfGroup($dbconnection,$group_id,$userid)
	{
		$treeview = self::getEgideBoxTreeView($dbconnection,$userid,$group_id);
		$box_list = self::getTreeViewBox($treeview);

		$tmp = array();
		foreach($box_list as $box)
		{
			$tmp[$box['id']] = $box;
		}
		return array_values($tmp);
	}

	public static function getDevicesOfAgentGroup($dbconnection,$group_id,$userid)
	{
		$box_list = self::getAgentsOfGroup($dbconnection,$group_id,$userid);

		$devices_list = array();
		foreach($box_list as $box)
		{
			$devices_list = array_merge($devices_list,self::getDevicesOfAgent($dbconnection,$box['id']));
		}
		return $devices_list;
	}

	public static function getDevices($dbconnection)
	{
		$sql = "SELECT eg_service_id, eg_device_id FROM public.egide_services;";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$row=$query->fetchAll(PDO::FETCH_ASSOC);
		if(empty($row))
		{
			$row = 'NONE';
		}

		return $row;
	}

	public static function getDeviceId($dbconnection,$dvc_name=NULL)
	{
		$sql = "SELECT device_id FROM public.esia_device WHERE device_name = :dvc_name";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($dvc_name));
		$row=$query->fetch(PDO::FETCH_OBJ);
		$dv_id = $row->device_id ;
		if(empty($dv_id))
		{
			$dv_id = 'NONE';
		}

		return $dv_id ;
	}

	public static function getCVE_Id($dbconnection,$cve_list=NULL)
	{
		$sql = "SELECT vul_id FROM public.egide_vulnerability WHERE vul_list_id = :cve_list";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($cve_list));
		$row=$query->fetch(PDO::FETCH_OBJ);
		$vl_id = $row->vul_id ;
		if(empty($vl_id))
		{
			$vl_id = 'NONE';
		}

		return $vl_id ;
	}

	private static function executeAndparseScannerHistoryRequest($dbconnection,$sql,$params,$with_devices)
	{
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$retRow=array(
				'id'=>$row->scan_history_id,
				'date_start'=>$row->scan_date_start,
				'date_end'=>$row->scan_date_end,
				'scan_state'=>$row->scan_state,
				'scan_id'=>$row->scan_id,
				'result'=>$row->scan_result
			);
			if( $with_devices )
			{
				// recherche du groupe name
				$option = $row->scan_option ;
				$option = json_decode($option);
				$group_node = $option->{'group_list'};
				$group_id = $group_node[0];
				$group = Model_esia::getGroupDeviceIn($dbconnection, $group_id);
				$group = array_shift($group);
				$groupsName = $group['name'];

				$retRow = array_merge($retRow,array(
					'iface_name'=>$row->vul_iface_local_name,
					'device_group_name'=>$row->device_group_name,
					'device_name'=>$row->device_name,
					'group_name' => $groupsName
				));
			}
			$arr[]=$retRow;
		}
		return $arr;
	}

	public static function getScannerHistory($dbconnection,$id)
	{
		$params = array();
		$sql = "SELECT scan_history_id,scan_date_start,scan_date_end,scan_id,scan_state,scan_result
						FROM vulnerability_scanner_history
						WHERE scan_history_id = :id";

		$ret = self::executeAndparseScannerHistoryRequest($dbconnection,$sql,array(':id'=>$id),FALSE);
		return array_shift($ret);
	}

	public static function getVulnerability_scanner_history($dbconnection,$agent_id = NULL,$limit = NULL)
	{
		$params = array();
		$sql = "
		SELECT scan_history_id,scan_date_start,scan_date_end,vulnerability_scanner_history.scan_id,vulnerability_scanner_history.scan_state,vulnerability_scanner.scan_option,scan_result,vul_iface_local_name,device_group_name,device_name
		FROM vulnerability_scanner_history
		JOIN vulnerability_scanner ON(vulnerability_scanner_history.scan_id = vulnerability_scanner.scan_id)
		JOIN vulnerability_iface ON(scan_iface_id = vul_iface_id)
		JOIN esia_device ON(esia_device.device_id = vulnerability_iface.vul_iface_agent_id)
		";

		if( $agent_id != NULL )
		{
			$in_request = generateInParams('WHERE',$agent_id,'vul_iface_agent_id',TRUE);
			$params = $in_request['param'];
			$where = $in_request['sql'];

			$sql = "$sql $where";
		}
		if( $limit != NULL )
		{
			$sql = "$sql ORDER BY scan_history_id DESC LIMIT :limit";
			$params[':limit'] = $limit;
		}
		return self::executeAndparseScannerHistoryRequest($dbconnection,$sql,$params,TRUE);
	}

	public static function getEgideServicesNode($dbconnection,$node_id,$iface_id = NULL)
	{
		$sql =  "SELECT eg_service_id, eg_service_name, eg_service_product, eg_service_version, eg_service_port, eg_service_cpe,
						eg_service_extrainfo,eg_service_port_type
				 FROM public.egide_services
				 JOIN egide_service_access ON (eg_access_service_id = eg_service_id)
				 WHERE eg_device_id = ?";

		$params = array($node_id);
		if( $iface_id  != NULL )
		{
			$sql.= " AND eg_access_iface_id = ?";
			$params[] = $iface_id;
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->eg_service_port]=array('id'=>$row->eg_service_id,
				'name'=>$row->eg_service_name,
				'product'=>$row->eg_service_product,
				'version'=>$row->eg_service_version,
				'port'=>$row->eg_service_port,
				'port_type'=>$row->eg_service_port_type,
				'cpe'=>$row->eg_service_cpe,
				'extrainfo'=>$row->eg_service_extrainfo);
		}
		return $arr;
	}

// 	public static function getServicesNode($dbconnection)
// 	{
// 		$sql = "SELECT eg_service_id, eg_service_name, eg_service_product, eg_service_version, eg_service_port, eg_service_cpe,
// 					eg_service_extrainfo,eg_service_port_type, detect_vul_id,vul_scoring, device_ip_address, device_name
// 				FROM public.egide_services
// 				JOIN egide_vulnerability_detect ON eg_service_id = detect_eg_service_id
// 				JOIN esia_device ON device_id = eg_device_id
// 				JOIN egide_vulnerability ON vul_id = detect_vul_id";
// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query);
// 		$arr = array();
// 		while($row=$query->fetch(PDO::FETCH_OBJ))
// 		{
// 			$arr[]=array('id'=>$row->eg_service_id,
// 				'name'=>$row->eg_service_name,
// 				'product'=>$row->eg_service_product,
// 				'version'=>$row->eg_service_version,
// 				'port'=>$row->eg_service_port,
// 				'port_type'=>$row->eg_service_port_type,
// 				'cpe'=>$row->eg_service_cpe,
// 				'vul_id'=>$row->detect_vul_id,
// 				'devc_ip'=>$row->device_ip_address,
// 				'devc_name'=>$row->device_name,
// 				'vul_score'=>$row->vul_scoring,
// 				'extrainfo'=>$row->eg_service_extrainfo);
// 		}
// 		return $arr;
// 	}

	public static function getServicesByCVE($dbconnection)
	{
		$sql = "SELECT eg_service_name,vul_list_id
				FROM public.egide_services
				JOIN egide_vulnerability_detect ON eg_service_id = detect_eg_service_id
				JOIN egide_vulnerability ON vul_id = detect_vul_id";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[]=array(
				'cve'=>$row->vul_list_id,
				'name'=>$row->eg_service_name
			);
		}
		return $arr;
	}

	public static function getCVEByNodes($dbconnection,$nodes)
	{
		$sql = "SELECT distinct vul_list_id, count(vul_list_id) as match, vul_description, vul_scoring, vul_severity, vul_attack_complexity
				FROM public.egide_vulnerability
				JOIN egide_vulnerability_detect ON vul_id = detect_vul_id
				JOIN egide_services ON eg_service_id = detect_eg_service_id
				where";
		foreach ($nodes as $node)
		{
			$sql=$sql." eg_device_id = ".$node." OR " ;
		}
		$sql = substr($sql,0,-3);
		$sql=$sql."group by vul_list_id,vul_description,vul_scoring,vul_severity,vul_attack_complexity" ;
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->vul_list_id]=array(
				'cve'=>$row->vul_list_id,
				'cve_descr'=>$row->vul_description,
				'cve_score'=>$row->vul_scoring,
				'cve_severt'=>$row->vul_severity,
				'cve_cmplx'=>$row->vul_attack_complexity,
				'match'=>$row->match,
			);
		}
		return $arr;
	}

	public static function getCVEByServices($dbconnection,$service_id)
	{
		$sql = "SELECT distinct vul_list_id, vul_date_publishing, vul_description, vul_scoring, vul_severity, vul_attack_complexity
				FROM public.egide_vulnerability
				JOIN egide_vulnerability_detect ON vul_id = detect_vul_id
				JOIN egide_services ON eg_service_id = detect_eg_service_id
				where eg_service_id = ".$service_id;

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->vul_list_id]=array(
				'id'=>$row->vul_list_id,
				'cve_pub'=>$row->vul_date_publishing,
				'description'=>$row->vul_description,
				'scoring'=>$row->vul_scoring,
				'severity'=>$row->vul_severity,
				'complexity'=>$row->vul_attack_complexity,
				'css'=> getCSSClassByscore($row->vul_scoring)
			);
		}
		return $arr;
	}

	public static function insertEgideServicesNode($dbconnection,$node,$name,$product,$version,$port,$type,$cpe,$extra,$iface_id)
	{
		$sql =  "INSERT INTO public.egide_services(eg_service_id,eg_service_name, eg_service_product, eg_service_version,
						eg_service_port,eg_service_port_type, eg_service_cpe, eg_service_extrainfo, eg_device_id)
				VALUES (DEFAULT,:name,:product,:version,:port,:type,:cpe,:extra,:node) RETURNING eg_service_id ;";
		$query = $dbconnection->prepare($sql);
		$params = array(':node'=>$node,
			':name'=>$name,
			':product'=>$product,
			':version'=>$version,
			':port'=>$port,
			':type'=>$type,
			':cpe'=>$cpe,
			':extra'=>$extra
		);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetch(PDO::FETCH_OBJ);

		self::linkServiceToIface($dbconnection,$iface_id,$row->eg_service_id);
		Model_esia::addNodeTabs($dbconnection, $node, 'TAB_EGIDESCAN');
		return $row->eg_service_id;
	}

	public static function linkServiceToIface($dbconnection,$iface_id,$service_id)
	{
		$sql = "INSERT INTO egide_service_access (eg_access_iface_id,eg_access_service_id)";
		$sql.= " VALUES(:iface_id,:service_id) ON CONFLICT (eg_access_iface_id,eg_access_service_id) DO NOTHING";
		$query = $dbconnection->prepare($sql);
		$params[':iface_id'] = $iface_id;
		$params[':service_id'] = $service_id;
		$query = Model_esia::executeStatement($query,$params);
	}

	public static function updateEgideServicesNode($dbconnection,$id,$name,$product,$version,$cpe,$extra,$iface_id)
	{
		$sql =  "UPDATE public.egide_services
				SET eg_service_name=:name,
					eg_service_product=:product,
					eg_service_version=:version,
					eg_service_cpe=:cpe,
					eg_service_extrainfo=:extra
				WHERE  eg_service_id=:id;";
		$query = $dbconnection->prepare($sql);
		$params = array(':id'=>$id,
			':name'=>$name,
			':product'=>$product,
			':version'=>$version,
			':cpe'=>$cpe,
			':extra'=>$extra
		);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetch(PDO::FETCH_OBJ);
		self::linkServiceToIface($dbconnection,$iface_id,$id);
		return $id;
	}
	public static function insertVulnerabilityDetect($dbconnection,$serviceid,$provider, $vulid, $scan_date)
	{
		//récupération du provider
		$sql= "SELECT vul_id, vul_provider_id, vul_list_id	FROM egide_vulnerability";
		if(is_string($provider))
		{
			$sql.=" WHERE vul_provider_id IN (SELECT provider_id FROM egide_provider WHERE provider_name = :provider)";

		}
		else
		{
			$sql.=" WHERE vul_provider_id = :provider";
		}
		$params[":provider"]=$provider;
		//récupération des CVEID
		if(is_array($vulid))
		{
			$arrids = implode("','", $vulid);
			$sql.= " AND vul_list_id IN ('$arrids')";
		}
		else
		{
			$sql.= " AND vul_list_id = :vulid";
			$params[":vulid"]=$vulid;
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		//préparation d'une reqûete d'insertion
		$queryinsert = $dbconnection->prepare("INSERT INTO egide_vulnerability_detect(detect_eg_service_id, detect_vul_id, detect_date,detect_date_update_state , detect_state)
				 VALUES (:serviceid,:vulid,to_timestamp(:detect_date),to_timestamp(:detect_date),'ACTIVE') ON CONFLICT ON CONSTRAINT \"detect_fk_uniq\"
				 	DO UPDATE SET detect_date_update_state=to_timestamp(:detect_date),detect_state = 'ACTIVE';");
		$inserparams[':serviceid']=$serviceid;
		$inserparams[':detect_date'] = $scan_date;

		$i=0;
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$inserparams[':vulid']=$row->vul_id;
			$queryinsert = Model_esia::executeStatement($queryinsert,$inserparams);
			$i++;
		}
		return $i;
	}

	public static function updateNotDetected($dbconnection,$serviceid, $scan_date)
	{
		$params[':detect_date'] = $scan_date;
		$params[':service_id'] = $serviceid;

		$sql = "UPDATE egide_vulnerability_detect
				SET detect_state = 'CLOSED',detect_date_update_state = to_timestamp(:detect_date)
				WHERE detect_state != 'CLOSED'
				AND detect_date != to_timestamp(:detect_date)
				AND detect_date_update_state != to_timestamp(:detect_date)
				AND detect_eg_service_id = :service_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		return $query->rowCount();
	}

	//mise à jour :

	public static function getEgideVulDetect($dbconnection)
	{
		$sql =  "SELECT detect_eg_service_id as detect_eg_service_id1, detect_vul_id as detect_vul_id1
	FROM public.egide_vulnerability_detect;";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$row=$query->fetchAll(PDO::FETCH_ASSOC);
		if(empty($row))
		{
			$row = 'NONE';
		}

		return $row;
	}

	public static function getCVEVulDetect($dbconnection,$agent_id=NULL)
	{
		$sql = "SELECT count(detect_vul_id) as match,vul_list_id,vul_description, vul_scoring, vul_severity, vul_attack_complexity";
		$sql.= " FROM egide_vulnerability LEFT JOIN egide_vulnerability_detect";
		$sql.= " ON (vul_id = detect_vul_id)";
		$sql.= " WHERE detect_state = 'ACTIVE'";

		$params = array();
		if( $agent_id != NULL )
		{
			$sub_req = "
				SELECT eg_service_id
				FROM egide_services
				INNER JOIN egide_service_access ON (eg_access_service_id = eg_service_id)
				INNER JOIN vulnerability_iface ON (vul_iface_id = eg_access_iface_id)
				WHERE vul_iface_agent_id = :agent_id
			";
			$sql.= "AND detect_eg_service_id IN($sub_req)";
			$params[':agent_id'] = $agent_id;
		}
		$sql.= " GROUP BY vul_list_id,vul_description, vul_scoring, vul_severity, vul_attack_complexity";
		$sql.= " ORDER BY vul_list_id DESC";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[]=array(
				'cve'=>$row->vul_list_id,
				'cve_descr'=>$row->vul_description,
				'cve_score'=>$row->vul_scoring,
				'cve_severt'=>$row->vul_severity,
				'cve_cmplx'=>$row->vul_attack_complexity,
				'match_vul'=> $row->match
			);
		}
		return $arr;
	}

	public static function getCVEVulDetectByIface($dbconnection,$iface_id)
	{
		$sql = "
			SELECT count(detect_vul_id) as match,vul_list_id, vul_scoring, vul_severity, vul_attack_complexity
			FROM vulnerability_iface
			INNER JOIN egide_service_access ON (vul_iface_id = eg_access_iface_id)
			INNER JOIN egide_services ON (eg_access_service_id = eg_service_id)
			LEFT JOIN egide_vulnerability_detect ON (eg_service_id = detect_eg_service_id)
			INNER JOIN egide_vulnerability ON (vul_id = detect_vul_id)
			WHERE vul_iface_id = :iface AND detect_state = 'ACTIVE'
			GROUP BY vul_list_id,vul_description, vul_scoring, vul_severity, vul_attack_complexity
			ORDER BY vul_list_id DESC;
		";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':iface'=>$iface_id));
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[]=array(
				'cve'=>$row->vul_list_id,
				'cve_score'=>$row->vul_scoring,
				'cve_severt'=>$row->vul_severity,
				'cve_cmplx'=>$row->vul_attack_complexity,
				'match_vul'=> $row->match
			);
		}
		return $arr;
	}

	public static function getCVE($dbconnection,$id_vul)
	{
		$sql =  "SELECT vul_list_id, vul_description, vul_scoring, vul_severity, vul_attack_complexity
				 FROM public.egide_vulnerability
				 WHERE vul_id = ?";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($id_vul));
		//$arr = array();

		$row=$query->fetch(PDO::FETCH_OBJ);
		$arr=array('cve'=>$row->vul_list_id,
			'cve_descr'=>$row->vul_description,
			'cve_score'=>$row->vul_scoring,
			'cve_severt'=>$row->vul_severity,
			'cve_cmplx'=>$row->vul_attack_complexity
		);
		return $arr;
	}

	public static function getServices_infos($dbconnection,$srvcId)
	{
		$sql =  "SELECT eg_service_name, eg_service_port, eg_service_product, eg_service_port_type
				 FROM public.egide_services
				 WHERE eg_service_id = ?";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($srvcId));
		//$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr=array('srvc_name'=>$row->eg_service_name,
				'srvc_port'=>$row->eg_service_port,
				'srvc_product'=>$row->eg_service_product,
				'srvc_port_type'=>$row->eg_service_port_type
			);
		}
		return $arr;
	}

	public static function getDetectCVEid($dbconnection,$detect_eg_serv_id=NULL)
	{
		$sql =  "SELECT detect_vul_id as detect_vul_id1
				 FROM public.egide_vulnerability_detect
				 WHERE detect_eg_service_id = :detect_eg_serv_id";
		$params = array(
			':detect_eg_serv_id' => $detect_eg_serv_id
		);
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetchAll(PDO::FETCH_ASSOC) ;
		//$detect_vul_id = $row->detect_vul_id1;
		if(empty($detect_vul_id))
		{
			$detect_vul_id = 'NONE';
		}

		return $row;
	}

	public static function getDetectCVEservice($dbconnection,$detect_vl_id=NULL)
	{
		$sql =  "SELECT detect_eg_service_id
				 FROM public.egide_vulnerability_detect
				 WHERE detect_vul_id = :detect_vl_id";
		$params = array(
			':detect_vl_id' => $detect_vl_id
		);
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetchAll(PDO::FETCH_ASSOC) ;
		//$detect_vul_id = $row->detect_vul_id1;
		if(empty($row))
		{
			$row = 'NONE';
		}

		return $row;
	}

	public static function getDevice_services($dbconnection,$srvc=NULL)
	{
		$sql =  "SELECT device_name
				 FROM public.esia_device
				 WHERE device_id=(SELECT eg_device_id FROM public.egide_services WHERE eg_service_id = :srvc) ";
		$params = array(
			':srvc' => $srvc
		);
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetch(PDO::FETCH_OBJ) ;
		$dvc_name = $row->device_name;
		if(empty($dvc_name))
		{
			$dvc_name = 'NONE';
		}

		return $dvc_name;
	}

	public static function updateVulnerabilityInfo($dbconnection,$vulUrl=NULL,$vulScore=NULL,$vulSeverity=NULL,$vulComplexity=NULL,$idvulnerability=NULL)
	{
		//$sql = "INSERT INTO public.egide_vulnerability(vul_url, vul_scoring, vul_severity, vul_attack_complexity) VALUES (:vulUrl , :vulScore, :vulSeverity, :vulComplexity);";
		$sql = "UPDATE public.egide_vulnerability
						SET vul_url= :vulUrl,
							vul_scoring= :vulScore,
							vul_severity= :vulSeverity,
							vul_attack_complexity= :vulComplexity
					WHERE vul_id = :idvulnerability;";
		$params = array(
			':vulUrl' =>$vulUrl,
			':vulScore' =>$vulScore,
			':vulSeverity' =>$vulSeverity,
			':vulComplexity' =>$vulComplexity,
			':idvulnerability' =>$idvulnerability
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
	}

	// scan config
	public static function Lastscan_id($dbconnection)
	{
		$sql0 = "SELECT max(scan_id) as lastscan FROM public.vulnerability_scanner;";
		$query0 = $dbconnection->prepare($sql0);
		$query0 = Model_esia::executeStatement($query0);
		$access = $query0->fetch(PDO::FETCH_OBJ);
		$id_lastScan = $access->lastscan;
		if(empty($id_lastScan) && ($id_lastScan!=0))
		{
			$id_lastScan = 0;
			return $id_lastScan ;
		}
		else
		{
			//$id_lastScan++;
			return $id_lastScan ;
		}
	}

	public static function addScan($dbconnection,$start_date,$start_time,$interval_value,$interval_unit,$iface_id,$list_scan,$retries,$retries_interval)
	{
		$cron_plugin = Model_esia::getCronPlugin($dbconnection,Model_egidescan::PLUGIN_CRON_START);
		$cron_plugin = array_shift($cron_plugin);

		$sql = 'INSERT INTO vulnerability_scanner';
		$sql.= ' (scan_iface_id,scan_option)';
		$sql.= " VALUES (:iface,:options)";
		$sql.= ' RETURNING scan_id';

		//Calcul de date
		$dt = DateTime::createFromFormat('d/m/Y H:i:s',"$start_date 00:00:00");
		$dt->add(new DateInterval("PT$start_time".'S'));

		//Création des données pour le scan_option json
		$options['group_list'] = $list_scan;
		$options['interval_value'] = $interval_value;
		$options['interval_unit'] = $interval_unit;

		//Configuration des paramètres
		$params[':iface'] = $iface_id;
		$params[':options'] = json_encode($options);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetch(PDO::FETCH_OBJ);

		//Configuration de la tâche cron
		$scan_id = $row->scan_id;

		switch($interval_unit)
		{
			case 'hour' :
				$interval_seconds = $interval_value * 3600;
				break;
			case 'day' :
				$interval_seconds = $interval_value * 86400;
				break;
			case 'week' :
				$interval_seconds = $interval_value * 86400;
				break;
			case 'month' :
				$startdate = new DateTime();
				$enddate = new DateTime();
				$startdate->sub(new DateInterval('P'.$interval_value.'M'));
				$startdate = $startdate->format('U');
				$enddate = $enddate->format('U');
				$interval_seconds = $enddate - $startdate;
				break;
		}

		$cron_id = Model_esia::addCronTask($dbconnection,$cron_plugin['id'],"-m start -i $scan_id",$interval_seconds,$dt->format('U'),"start_scan_$scan_id_".'$DATE.log',$retries,$retries_interval);

		$params = array();
		$sql = "UPDATE vulnerability_scanner
				SET scan_cron_exec = :cron_id
				WHERE scan_id = :scan_id";


		$params[':cron_id'] = $cron_id;
		$params[':scan_id'] = $scan_id;
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		return $scan_id;
	}

	public static function removeScan($dbconnection,$scan_id)
	{
		//TODO : Remplacer la recherche par paramètre par une recherche en ajoutant une clé étrangere dans egide_scan
		$cron_plugin = Model_esia::getCronPlugin($dbconnection,Model_egidescan::PLUGIN_CRON_START);
		$cron_plugin = array_shift($cron_plugin);
		$cron_list = Model_esia::getCronActiveTasks($dbconnection,NULL,$cron_plugin['id']);

		$found_cron_task = NULL;
		foreach($cron_list as $cron)
		{
			if( $cron['parameter'] == "-m start -i $scan_id")
			{
				$found_cron_task = $cron;
			}
		}

		if( $found_cron_task != NULL )
		{
			Model_esia::deleteCronTask($dbconnection,$found_cron_task['id']);
		}

		$params =  array();
		$params[':scan_id'] = $scan_id;
		$sql_history = 'DELETE FROM vulnerability_scanner_history WHERE scan_id = :scan_id';
		$sql_scanner = 'DELETE FROM vulnerability_scanner WHERE scan_id = :scan_id';

		$query = $dbconnection->prepare($sql_history);
		$query = Model_esia::executeStatement($query,$params);

		$query = $dbconnection->prepare($sql_scanner);
		$query = Model_esia::executeStatement($query,$params);
	}

	public static function updateScan($dbconnection,$scan_id,$start_date,$start_time,$interval_value,$interval_unit,$iface_id,$list_scan,$retries,$retries_interval)
	{
		$params =  array();
		$params[':scan_id'] = $scan_id;
		$sql = "
			UPDATE vulnerability_scanner
			SET scan_iface_id=:iface,
				scan_option=:options
			WHERE scan_id = :scan_id
		";

		//Calcul de date
		$dt = DateTime::createFromFormat('d/m/Y H:i:s',"$start_date 00:00:00");
		$dt->add(new DateInterval("PT$start_time".'S'));

		//Création des données pour le scan_option json
		$options['group_list'] = $list_scan;
		$options['interval_value'] = $interval_value;
		$options['interval_unit'] = $interval_unit;

		//Configuration des paramètres
		$params[':iface'] = $iface_id;
		$params[':options'] = json_encode($options);

		//TODO : Remplacer la recherche par paramètre par une recherche en ajoutant une clé étrangere dans egide_scan
		$cron_plugin = Model_esia::getCronPlugin($dbconnection,Model_egidescan::PLUGIN_CRON_START);
		$cron_plugin = array_shift($cron_plugin);
		$cron_list = Model_esia::getCronActiveTasks($dbconnection,NULL,$cron_plugin['id']);

		$found_cron_task = NULL;
		foreach($cron_list as $cron)
		{
			if( $cron['parameter'] == "-m start -i $scan_id")
			{
				$found_cron_task = $cron;
			}
		}

		if( $found_cron_task != NULL )
		{
			switch($interval_unit)
			{
				case 'hour' :
					$interval_seconds = $interval_value * 3600;
					break;
				case 'day' :
					$interval_seconds = $interval_value * 86400;
					break;
				case 'week' :
					$interval_seconds = $interval_value * 86400;
					break;
				case 'month' :
					$startdate = new DateTime();
					$enddate = new DateTime();
					$startdate->sub(new DateInterval('P'.$interval_value.'M'));
					$startdate = $startdate->format('U');
					$enddate = $enddate->format('U');
					$interval_seconds = $enddate - $startdate;
					break;
			}
			Model_esia::updateCronTask($dbconnection,$found_cron_task['id'],"-m start -i $scan_id",$interval_seconds,$dt->format('U'),"start_scan_$scan_id_".'$DATE.log',$retries,$retries_interval);
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		return $query->rowCount();
	}

	public static function getScanList($dbconnection,$id=NULL,$agent_id=NULL)
	{
		$sql = "SELECT
			scan_id,
			cron_last_exec,
			cron_next_exec,
			scan_state,
			scan_option,
			vul_iface_id,
			vul_iface_local_name,
			vul_iface_label,
			device_id,
			device_name,
			device_ip_address,
			vul_type_name,
			vul_type_color,
			vul_type_factor
			FROM vulnerability_scanner
			INNER JOIN vulnerability_iface ON(scan_iface_id = vul_iface_id)
			INNER JOIN esia_device ON(vul_iface_agent_id = device_id)
			INNER JOIN vulnerability_type ON(vul_type_id = vul_iface_type_id)
			INNER JOIN esia_cron_plugin ON (scan_cron_exec = cron_id)
			";

		if( $id != NULL )
		{
			$sql.= " WHERE scan_id = :scan_id";
			$params[':scan_id'] = $id;
		}
		elseif( $agent_id != NULL )
		{
			$in_request = generateInParams('WHERE',$agent_id,'vul_iface_agent_id',TRUE);
			$params = $in_request['param'];
			$where = $in_request['sql'];

			$sql = "$sql $where";
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$retRow = array(
				'id' => $row->scan_id,
				'state' => $row->scan_state,
				'last_exec' => $row->cron_last_exec,
				'next_exec' => $row->cron_next_exec,
				'options' => json_decode($row->scan_option,TRUE),
				'iface_id' => $row->vul_iface_id,
				'iface' => $row->vul_iface_local_name,
				'label' => $row->vul_iface_label,
				'agent_id'=> $row->device_id,
				'agent_name'=> $row->device_name,
				'agent_addr'=> $row->device_ip_address,
				'type_name' => $row->vul_type_name,
				'type_color' => $row->vul_type_color,
				'type_factor' => $row->vul_type_factor,
			);

			if( empty($retRow['state']))
			{
				$retRow['state'] = self::NEW_TASK;
			}
			$ret[] = $retRow;
		}
		return $ret;
	}

	public static function getScanById($dbconnection,$id)
	{
		$list = self::getScanList($dbconnection,$id);
		return array_shift($list);
	}

	public static function getAgentScanList($dbconnection,$agent)
	{
		return self::getScanList($dbconnection,NULL,$agent);
	}

	public static function getNodesOfGroupList($dbconnection,$group_list)
	{
		$all_node_list = array();
		foreach($group_list as $group_id)
		{
			$nodes_group = Model_esia::newgetNodeList($dbconnection,NULL,$group_id,Model_esia::NODE_INSTANCE);
			//On ne souhaite qu'un seul exemplaire à la fois du même noeud
			foreach($nodes_group as $node)
			{
				$all_node_list[$node->getID()] = $node;
			}
		}
		return array_values($all_node_list);
	}

	public static function getCurrentScanList($dbconnection,$status=NULL)
	{
		$sql = "SELECT scan_id,scan_state,scan_option,vul_iface_local_name,device_id,device_ip_address";
		$sql.= " FROM vulnerability_scanner INNER JOIN vulnerability_iface";
		$sql.= " ON(scan_iface_id = vul_iface_id)";
		$sql.= " INNER JOIN esia_device";
		$sql.= " ON(vul_iface_agent_id = device_id)";
		$sql.= " INNER JOIN esia_cron_plugin ON (scan_cron_exec = cron_id)";
		$params = array();

		if( $status != null )
		{
			if( $status == self::PENDING_TASK )
			{
				$sql.= ' AND (scan_state = :status OR scan_state IS NULL)';
			}
			else
			{
				$sql.= ' AND scan_state = :status';
			}
			$params[':status'] = $status;
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array(
				'id' => $row->scan_id,
				'state' => $row->scan_state,
				'options' => json_decode($row->scan_option,TRUE),
				'iface' => $row->vul_iface_local_name,
				'agent_id'=> $row->device_id,
				'agent_addr'=> $row->device_ip_address
			);
		}
		return $arr;
	}

	public static function updateScanningTaskStatus($dbconnection,$scan_id,$status)
	{
		$sql = "UPDATE vulnerability_scanner SET scan_state=:state";

		$params[':id'] = $scan_id;
		$params[':state'] = $status;

		$sql.= " WHERE scan_id=:id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
	}

// 	public static function newScan($dbconnection, $id_lastScan=NULL, $scan_date=NULL,$dv_scanned=NULL,$interval_exec=NULL,$interface=NULL)
// 	{
// 		$sql = "SELECT vul_iface_id FROM vulnerability_iface WHERE vul_iface_agent_id = :dv_scanned AND $interface = :interface";
// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query, array(':dv_scanned'=>$dv_scanned,':interface'=>$interface));
// 		$row = $query->fetch(PDO::FETCH_ASSOC);

// 		$sql = "INSERT INTO public.vulnerability_scanner(scan_id, scan_next_execution, scan_agent_id,scan_interval_execution,scan_src_interface)
//                                         VALUES (:scan_id, :scan_date, :interval_exec, :scan_iface_id);";
// 		$params = array(
// 			':scan_id' =>$id_lastScan,
// 			':scan_date' =>$scan_date,
// 			':interval_exec' =>$interval_exec,
// 			':scan_iface_id' =>$row['vul_iface_id']
// 		);

// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query, $params);
// 	}

	public static function getConfigScan($dbconnection,$id_lastScan=NULL)
	{
		$sql = "SELECT cron_next_exec,vul_iface_agent_id";
		$sql.= "\nFROM vulnerability_scanner INNER JOIN vulnerability_iface";
		$sql.= "\nON(scan_iface_id = vul_iface_type_id) INNER JOIN esia_cron_plugin ON (scan_cron_exec = cron_id)";
		$sql.= "\nWHERE scan_id= :id_lastScan;";

		$ret = array('NONE','NONE');
		if( $id_lastScan != NULL )
		{
			$params = array(
				':id_lastScan' => $id_lastScan
			);

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query, $params);
			$row = $query->fetch(PDO::FETCH_ASSOC);

			if(!empty($row['cron_next_exec']))
			{
				$ret[0] = $row['cron_next_exec'];
			}
			if(!empty($row['cron_next_exec']))
			{
				$ret[1] = $row['vul_iface_agent_id'];
			}
		}
		return $ret;
	}

	public static function getInterfaceScan($dbconnection,$id_lastScan=NULL)
	{
		$sql = "SELECT vul_iface_local_name";
		$sql.= "\nFROM vulnerability_scanner INNER JOIN vulnerability_iface";
		$sql.= "\nON(scan_iface_id = vul_iface_type_id)";
		$sql.= "\nWHERE scan_id= :id_lastScan;";

		$ret = 'NONE';
		if( $id_lastScan != NULL )
		{
			$params = array(
				':id_lastScan' => $id_lastScan
			);

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query, $params);
			$row = $query->fetch(PDO::FETCH_ASSOC);

			if(!empty($row['vul_iface_local_name']))
			{
				$ret = $row['vul_iface_local_name'];
			}
		}
		return $ret;
	}
/*
	public static function updateNextExecution($dbconnection,$id_lastScan=NULL)
	{
		$sql0 = "SELECT scan_next_execution as next_exec, scan_interval_execution as interval_exec FROM public.vulnerability_scanner WHERE scan_id = :id_lastScan;";
		$params0 = array(
			':id_lastScan' => $id_lastScan
		);
		$query0 = $dbconnection->prepare($sql0);
		$query0 = Model_esia::executeStatement($query0, $params0);
		$access0 = $query0->fetch(PDO::FETCH_OBJ);
		$interval_exec = $access0->interval_exec;
		$next_exec = $access0->next_exec;
		if(empty($interval_exec))
		{
			$interval_exec = 'NONE';
		}


		//////
		// 		$next_exec = date_create($next_exec);
		// 		date_modify($next_exec, "$interval_exec");
		/////

		$sql = "UPDATE public.vulnerability_scanner
						SET scan_next_execution= scan_next_execution + :interval_exec
					WHERE scan_id = :id_lastScan;";
		$params = array(
			// 			':next_exec' =>date_format($next_exec, 'Y-m-d H:i:s'),
			':id_lastScan' => $id_lastScan,
			':interval_exec' => $interval_exec
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
	}
	*/
	public static function getNumberGlobalByScoring($dbconnection,$user, $node=null)
	{
		$params = array();
		$sql = "SELECT FLOOR(vul_scoring) as domain,count(*) as number
                FROM egide_vulnerability
				JOIN egide_vulnerability_detect  ON vul_id = detect_vul_id
				JOIN egide_services ON eg_service_id=detect_eg_service_id
                WHERE vul_scoring IS NOT NULL AND eg_device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND detect_state = 'ACTIVE'";
        $params[':user'] = $user;

		$in_request = generateInParams('AND',$node,'eg_device_id');
		$param = $in_request['param'];
		$where = $in_request['sql'];

		$sql.= ' ' . $where;
		$params = array_merge($params,$param);

        $sql .=" GROUP BY FLOOR(vul_scoring)";
        $query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('domain' => $row->domain,
				'number' => $row->number);
		}
		return $arr;
	}

	public static function getServicesByIDList($dbconnection,$user,$cve_list=NULL,$include_cve=TRUE)
	{
		$in_request = generateInParams('AND',$cve_list,'eg_service_id',$include_cve);
		$params = $in_request['param'];
		$where = $in_request['sql'];

		$sql = "SELECT eg_service_id, eg_service_name, eg_service_product, eg_service_version, eg_service_port, eg_service_cpe,
					eg_service_extrainfo,eg_service_port_type, device_ip_address, device_name, device_type,
					count(detect_vul_id) as matchnum,AVG(vul_scoring) AS avg_scoring,MAX(vul_scoring) AS max_scoring
				FROM public.egide_services
				JOIN egide_vulnerability_detect ON eg_service_id = detect_eg_service_id
				JOIN esia_device ON device_id = eg_device_id
				JOIN egide_vulnerability ON vul_id = detect_vul_id
				WHERE eg_device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND detect_state = 'ACTIVE' $where
				GROUP BY eg_service_id, eg_service_name, eg_service_product, eg_service_version, eg_service_port, eg_service_cpe,
					eg_service_extrainfo,eg_service_port_type, device_ip_address, device_name,device_type";

		$params[':user'] = $user;

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array(
							'id' => $row->eg_service_id,
							'name' => $row->device_name . ' '.$row->eg_service_product . ' port '.$row->eg_service_name . '('.$row->eg_service_port .')',
							'number' => $row->matchnum
							);
		}
		return $arr;
	}


	public static function getAcqCVE($dbconnection,$user,$cve_list=NULL,$include_cve=TRUE)
	{
		$sql = "
			SELECT vul_list_id, vul_description,vul_scoring, vul_severity, vul_attack_complexity, device_name,eg_service_name,detect_id
			FROM egide_vulnerability
			JOIN egide_vulnerability_detect  ON vul_id = detect_vul_id
			JOIN egide_services ON eg_service_id=detect_eg_service_id
			JOIN esia_device ON eg_device_id=device_id
			WHERE eg_device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND detect_state = 'ACQUIT'
		";
		$params[':user'] = $user;

		$in_request = generateInParams('AND',$cve_list,'detect_id',$include_cve);
		$param = $in_request['param'];
		$where = $in_request['sql'];

		$sql.= ' ' . $where;
		$params = array_merge($params,$param);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array(
							'id' => $row->detect_id,
							'id_vul' => $row->vul_list_id,
                            'description' => $row->vul_description,
                            'scoring' => $row->vul_scoring,
                            'severity' => $row->vul_severity,
                            'complexity' => $row->vul_attack_complexity,
							'device_name' => $row->device_name,
							'service_name' => $row->eg_service_name
                            );
        }
        return $arr;
	}

    public static function getCVEByScoring($dbconnection,$user, $node=null,$limit=null,$cve_list=NULL,$include_cve=TRUE)
    {
		//Préparation de la requête
		$params = array();
		$sql_case = "
			CASE
				WHEN detect_state = 'ACTIVE' THEN '0_ACTIVE'
				WHEN detect_state = 'ACQUIT' THEN '1_ACQUIT'
				WHEN detect_state = 'CLOSED' THEN '2_CLOSED'
			END
		";

		$sql = "
			SELECT vul_list_id, vul_description,vul_scoring, vul_severity, vul_attack_complexity,count(detect_vul_id) as matchnum,
			    split_part(min($sql_case), '_',2) AS priority_detect_state
			FROM egide_vulnerability
			JOIN egide_vulnerability_detect  ON vul_id = detect_vul_id
			JOIN egide_services ON eg_service_id=detect_eg_service_id
			WHERE eg_device_id IN ( SELECT * FROM getUserAccessNode(:user))
		";
		$params[':user'] = $user;

		$in_request = generateInParams('AND',$node,'eg_device_id');
		$param = $in_request['param'];
		$where = $in_request['sql'];

		$sql.= ' ' . $where;
		$params = array_merge($params,$param);

		$in_request = generateInParams('AND',$cve_list,'vul_list_id',$include_cve);
		$param = $in_request['param'];
		$where = $in_request['sql'];

		$sql.= ' ' . $where;
		$params = array_merge($params,$param);

		$sql .= " GROUP BY vul_list_id,vul_description, vul_scoring, vul_severity, vul_attack_complexity, detect_state
					ORDER BY ($sql_case) ASC,vul_scoring DESC";

		//Récupération des CVE en cours
		if(!empty($limit))
        {
            $sql .=" LIMIT :limit";
            $params[':limit'] = $limit;
        }



        $query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id' => $row->vul_list_id,
                            'description' => $row->vul_description,
                            'scoring' => $row->vul_scoring,
                            'severity' => $row->vul_severity,
                            'complexity' => $row->vul_attack_complexity,
                            'number' => $row->matchnum,
							'state' => $row->priority_detect_state
                            );
        }
        return $arr;
    }
		public static function getNodesByScoring($dbconnection,$user, $node=null,$limit=null)
    	{
        	$params = array();

			if( is_array($node) || is_numeric($node))
			{
				$in_request = generateInParams('AND',$node,'device_id');
				$param = $in_request['param'];
				$where = $in_request['sql'];
				$params = array_merge($params,$param);
			}
			else
			{
				$where = "AND device_name = :name";
				$params[':name'] = $node;
			}

				$sql = "
					(
						With active_vul AS (
							SELECT device_id,device_name,device_description,device_type,device_ip_address,
								COUNT(detect_vul_id) as number,
								MAX(vul_scoring) as maximun,
								SUM(vul_scoring) as total
							FROM esia_device
							JOIN egide_services ON device_id=eg_device_id
							JOIN egide_vulnerability_detect ON eg_service_id = detect_eg_service_id
							JOIN egide_vulnerability ON vul_id = detect_vul_id
							WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND detect_state = 'ACTIVE' $where
							GROUP BY device_id,device_name,device_description,device_type,device_ip_address
						)
						SELECT device_id,device_name,device_description,device_type,device_ip_address,number,maximun,total
						FROM active_vul
						UNION
						SELECT device_id,device_name,device_description,device_type,device_ip_address,0 as number,0 as maximun,0 as total
						FROM esia_device
						WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND device_id NOT IN(SELECT device_id from active_vul) $where
					)
					ORDER BY maximun DESC,total DESC, number DESC, device_name ASC
				";
				$params[':user'] = $user;

        if(!empty($limit))
        {
            $sql .=" LIMIT :limit";
            $params[':limit'] = $limit;
        }
        $query = $dbconnection->prepare($sql);

		$query = Model_esia::executeStatement($query, $params);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$average_score = 0;
			if( $row->number > 0 )
			{
				$average_score = round($row->total/$row->number,1);
			}

			$arr[] = array('id' => $row->device_id,
                            'name' => $row->device_name,
                            'description' => $row->device_description,
                            'type' => $row->device_type,
                            'ip' => $row->device_ip_address,
                            'numberVulnerability' => $row->number,
                            'totalVulnerabilityScoring' => $row->total,
                            'averageVulnerabilityScoring' => $average_score,
                            'maxVulnerabilityScoring' => $row->maximun);
        }
        return $arr;
    }
}
?>
